@@ -5,7 +5,6 @@ from neo.SmartContract.Iterable import EnumeratorBase
 
 class DBCollection:
     DB = None
-    #    SN = None
     Prefix = None
 
     ClassRef = None
@@ -19,7 +18,10 @@ class DBCollection:
 
     DebugStorage = False
 
-    def __init__(self, db, sn, prefix, class_ref):
+    _ChangedResetState = None
+    _DeletedResetState = None
+
+    def __init__(self, db, prefix, class_ref):
 
         self.DB = db
 
@@ -31,6 +33,9 @@ class DBCollection:
         self.Changed = []
         self.Deleted = []
 
+        self._ChangedResetState = None
+        self._DeletedResetState = None
+
     @property
     def Keys(self):
         if not self._built_keys:
@@ -71,12 +76,15 @@ class DBCollection:
         else:
             self.Changed = []
             self.Deleted = []
+            self._ChangedResetState = None
+            self._DeletedResetState = None
 
     def Reset(self):
-        for keyval in self.Changed:
-            self.Collection[keyval] = None
-        self.Changed = []
-        self.Deleted = []
+        self.Changed = self._ChangedResetState
+        self.Deleted = self._DeletedResetState
+
+        self._ChangedResetState = None
+        self._DeletedResetState = None
 
     def GetAndChange(self, keyval, new_instance=None, debug_item=False):
 
@@ -173,6 +181,10 @@ class DBCollection:
         if keyval not in self.Deleted:
             self.Deleted.append(keyval)
 
+    def MarkForReset(self):
+        self._ChangedResetState = self.Changed
+        self._DeletedResetState = self.Deleted
+
     def MarkChanged(self, keyval):
         if keyval not in self.Changed:
             self.Changed.append(keyval)
@@ -206,10 +218,11 @@ class DBCollection:
 
     def Destroy(self):
         self.DB = None
-        #        self.SN = None
         self.Collection = None
         self.ClassRef = None
         self.Prefix = None
         self.Deleted = None
         self.Changed = None
+        self._ChangedResetState = None
+        self._DeletedResetState = None
         logger = None
