@@ -1,10 +1,10 @@
 import binascii
 import random
-import traceback
 from logzero import logger
 from twisted.internet.protocol import Protocol
 from twisted.internet import error as twisted_error
-from twisted.internet import reactor, task
+from twisted.internet import reactor, task, threads
+
 from neo.Core.Blockchain import Blockchain as BC
 from neocore.IO.BinaryReader import BinaryReader
 from neo.Network.Message import Message
@@ -29,9 +29,9 @@ class NeoNode(Protocol):
 
     leader = None
 
-    block_loop = None
+    block_loop_deferred = None
 
-    peer_loop = None
+    peer_loop_deferred = None
 
     sync_mode = MODE_CATCHUP
 
@@ -67,10 +67,10 @@ class NeoNode(Protocol):
 
     def Disconnect(self, reason=None):
         """Close the connection with the remote node client."""
+        self.expect_verack_next = False
         if reason:
             logger.debug(reason)
         self.transport.loseConnection()
-        self.expect_verack_next = False
 
     @property
     def Address(self):
@@ -124,20 +124,21 @@ class NeoNode(Protocol):
 
     def connectionLost(self, reason=None):
         """Callback handler from twisted when a connection was lost."""
-        if self.block_loop:
-            self.block_loop.stop()
-            self.block_loop = None
-        if self.peer_loop:
-            self.peer_loop.stop()
-            self.peer_loop = None
+        try:
+            if self.block_loop_deferred:
+                self.block_loop_deferred.cancel()
+            if self.peer_loop_deferred:
+                self.block_loop_deferred.cancel()
 
-        self.ReleaseBlockRequests()
-        self.leader.RemoveConnectedPeer(self)
+            self.ReleaseBlockRequests()
+            self.leader.RemoveConnectedPeer(self)
 
-        if reason and reason.check(twisted_error.ConnectionDone):
-            self.Log("client {} disconnected normally with reason:{}".format(self.remote_nodeid, reason))
-        else:
-            self.Log("%s disconnected %s" % (self.remote_nodeid, reason))
+            if reason and reason.check(twisted_error.ConnectionDone):
+                self.Log("client {} disconnected normally with reason:{}".format(self.remote_nodeid, reason))
+            else:
+                self.Log("%s disconnected %s" % (self.remote_nodeid, reason))
+        except Exception as e:
+            logger.error("Error with connection lost: %s " % e)
 
     def ReleaseBlockRequests(self):
         bcr = BC.Default().BlockRequests
@@ -213,7 +214,6 @@ class NeoNode(Protocol):
             self.MessageReceived(message)
 
         except Exception as e:
-            traceback.print_exc()
             self.Log('Error: Could not extract message: %s ' % e)
             return
 
@@ -257,25 +257,32 @@ class NeoNode(Protocol):
             self.HandleGetHeadersMessageReceived(m.Payload)
         elif m.Command == 'headers':
             reactor.callFromThread(self.HandleBlockHeadersReceived, m.Payload)
-        #            self.HandleBlockHeadersReceived(m.Payload)
+
         elif m.Command == 'addr':
             self.HandlePeerInfoReceived(m.Payload)
         else:
             self.Log("Command not implemented {} {} ".format(m.Command, self.endpoint))
 
+    def OnLoopError(self, err):
+        self.Log("On neo Node loop error %s " % err)
+
+    def onThreadDeferredErr(self, err):
+        logger.error("On Call from thread error %s " % err)
+
     def ProtocolReady(self):
+        block_loop = task.LoopingCall(self.AskForMoreBlocks)
+        self.block_loop_deferred = block_loop.start(self.sync_mode, now=False)
+        self.block_loop_deferred.addErrback(self.OnLoopError)
+
+        # ask every 3 minutes for new peers
+        peer_loop = task.LoopingCall(self.RequestPeerInfo)
+        self.peer_loop_deferred = peer_loop.start(120, now=False)
+        self.peer_loop_deferred.addErrback(self.OnLoopError)
+
         self.RequestPeerInfo()
         self.AskForMoreHeaders()
 
-        self.block_loop = task.LoopingCall(self.AskForMoreBlocks)
-        self.block_loop.start(self.sync_mode)
-
-        # ask every 3 minutes for new peers
-        self.peer_loop = task.LoopingCall(self.RequestPeerInfo)
-        self.peer_loop.start(120, now=False)
-
     def AskForMoreHeaders(self):
-        # self.Log("asking for more headers...")
         get_headers_message = Message("getheaders", GetBlocksPayload(hash_start=[BC.Default().CurrentHeaderHash]))
         self.SendSerializedMessage(get_headers_message)
 
@@ -291,9 +298,11 @@ class NeoNode(Protocol):
             self.sync_mode = MODE_MAINTAIN
 
         if self.sync_mode != current_mode:
-            self.block_loop.stop()
-            # self.Log("Changing sync mode from %s to %s" % (current_mode, self.sync_mode))
-            self.block_loop.start(self.sync_mode)
+            self.block_loop_deferred.cancel()
+            block_loop = task.LoopingCall(self.AskForMoreBlocks)
+            self.block_loop_deferred = block_loop.start(self.sync_mode)
+            self.block_loop_deferred.addErrback(self.OnLoopError)
+
         else:
             if len(BC.Default().BlockRequests) > self.leader.BREQMAX:
                 self.leader.ResetBlockRequestsAndCache()
@@ -334,7 +343,8 @@ class NeoNode(Protocol):
 
         if len(hashes) > 0:
             message = Message("getdata", InvPayload(InventoryType.Block, hashes))
-            reactor.callInThread(self.SendSerializedMessage, message)
+            sendDeferred = threads.deferToThread(self.SendSerializedMessage, message)
+            sendDeferred.addErrback(self.onThreadDeferredErr)
 
     def RequestPeerInfo(self):
         """Request the peer address information from the remote client."""
@@ -344,6 +354,9 @@ class NeoNode(Protocol):
         """Process response of `self.RequestPeerInfo`."""
         addrs = IOHelper.AsSerializableWithType(payload, 'neo.Network.Payloads.AddrPayload.AddrPayload')
 
+        if not addrs:
+            return
+
         for index, nawt in enumerate(addrs.NetworkAddressesWithTime):
             self.leader.RemoteNodePeerReceived(nawt.Address, nawt.Port, index)
 
@@ -364,7 +377,6 @@ class NeoNode(Protocol):
 
     def RequestVersion(self):
         """Request the remote client version."""
-        # self.Log("All caught up, requesting version")
         m = Message("getversion")
         self.SendSerializedMessage(m)
 
@@ -383,6 +395,9 @@ class NeoNode(Protocol):
         """Process the response of `self.RequestVersion`."""
         self.Version = IOHelper.AsSerializableWithType(payload, "neo.Network.Payloads.VersionPayload.VersionPayload")
 
+        if not self.Version:
+            return
+
         if self.incoming_client:
             if self.Version.Nonce == self.nodeid:
                 self.Disconnect()
@@ -409,6 +424,8 @@ class NeoNode(Protocol):
             return
 
         inventory = IOHelper.AsSerializableWithType(payload, 'neo.Network.Payloads.InvPayload.InvPayload')
+        if not inventory:
+            return
 
         if inventory.Type == InventoryType.BlockInt:
 
@@ -429,11 +446,6 @@ class NeoNode(Protocol):
         elif inventory.Type == InventoryType.ConsensusInt:
             pass
 
-    #        for hash in inventory.Hashes:
-    #            if not hash in self.leader.KnownHashes:
-    #                if not hash in self.leader.MissionsGlobal:
-    #                    self.leader.MissionsGlobal.append(hash)
-
     def SendSerializedMessage(self, message):
         """
         Send the `message` to the remote client.
@@ -441,10 +453,13 @@ class NeoNode(Protocol):
         Args:
             message (neo.Network.Message):
         """
-        ba = Helper.ToArray(message)
-        ba2 = binascii.unhexlify(ba)
-        self.bytes_out += len(ba2)
-        self.transport.write(ba2)
+        try:
+            ba = Helper.ToArray(message)
+            ba2 = binascii.unhexlify(ba)
+            self.bytes_out += len(ba2)
+            self.transport.write(ba2)
+        except Exception as e:
+            self.Log("Could not send serialized message %s " % e)
 
     def HandleBlockHeadersReceived(self, inventory):
         """
@@ -453,15 +468,19 @@ class NeoNode(Protocol):
         Args:
             inventory (neo.Network.Inventory):
         """
-        inventory = IOHelper.AsSerializableWithType(inventory, 'neo.Network.Payloads.HeadersPayload.HeadersPayload')
-        #        self.Log("Received headers %s " % ([h.Hash.ToBytes() for h in inventory.Headers]))
-        if inventory is not None:
-            BC.Default().AddHeaders(inventory.Headers)
+        try:
+            inventory = IOHelper.AsSerializableWithType(inventory, 'neo.Network.Payloads.HeadersPayload.HeadersPayload')
+            if inventory is not None:
+                BC.Default().AddHeaders(inventory.Headers)
 
-        if BC.Default().HeaderHeight < self.Version.StartHeight:
-            self.AskForMoreHeaders()
-        else:
-            reactor.callLater(5, self.AskForMoreHeaders)
+            if BC.Default().HeaderHeight < self.Version.StartHeight:
+                self.AskForMoreHeaders()
+            else:
+                deferredCall = task.deferLater(reactor, 5, self.AskForMoreHeaders)
+                deferredCall.addErrback(self.onThreadDeferredErr)
+
+        except Exception as e:
+            self.Log("Error handling Block headers %s " % e)
 
     def HandleBlockReceived(self, inventory):
         """
@@ -471,6 +490,8 @@ class NeoNode(Protocol):
             inventory (neo.Network.Inventory):
         """
         block = IOHelper.AsSerializableWithType(inventory, 'neo.Core.Block.Block')
+        if not block:
+            return
 
         blockhash = block.Hash.ToBytes()
         if blockhash in BC.Default().BlockRequests:
@@ -523,6 +544,8 @@ class NeoNode(Protocol):
             payload (neo.Network.Inventory):
         """
         inventory = IOHelper.AsSerializableWithType(payload, 'neo.Network.Payloads.InvPayload.InvPayload')
+        if not inventory:
+            return
 
         for hash in inventory.Hashes:
             hash = hash.encode('utf-8')
@@ -564,11 +587,12 @@ class NeoNode(Protocol):
             return
 
         inventory = IOHelper.AsSerializableWithType(payload, 'neo.Network.Payloads.GetBlocksPayload.GetBlocksPayload')
+        if not inventory:
+            return
 
         blockchain = BC.Default()
         hash = inventory.HashStart[0]
         if not blockchain.GetHeader(hash):
-            self.Log("Hash {} not found ".format(hash))
             return
 
         hashes = []
@@ -580,7 +604,6 @@ class NeoNode(Protocol):
             hashes.append(hash)
             hcount += 1
         if hcount > 0:
-            self.Log("sending inv hashes! %s " % hashes[0])
             self.SendSerializedMessage(Message('inv', InvPayload(type=InventoryType.Block, hashes=hashes)))
 
     def Relay(self, inventory):
