@@ -9,7 +9,6 @@ from neo.Network.NeoNode import NeoNode
 from neo.Settings import settings
 from twisted.internet.protocol import ReconnectingClientFactory
 from twisted.internet import reactor, task
-from twisted.internet.endpoints import TCP4ServerEndpoint
 
 
 class NeoClientFactory(ReconnectingClientFactory):
@@ -70,7 +69,7 @@ class NodeLeader:
 
     ServiceEnabled = False
 
-    peer_check_loop = None
+    peer_loop_deferred = None
 
     @staticmethod
     def Instance():
@@ -107,9 +106,9 @@ class NodeLeader:
         self.NodeId = random.randint(1294967200, 4294967200)
 
     def Restart(self):
-        if self.peer_check_loop:
-            self.peer_check_loop.stop()
-            self.peer_check_loop = None
+        if self.peer_loop_deferred:
+            self.peer_loop_deferred.cancel()
+            self.peer_loop_deferred = None
 
         if len(self.Peers) == 0:
             self.ADDRS = []
@@ -118,16 +117,17 @@ class NodeLeader:
 
     def Start(self):
         """Start connecting to the node list."""
-        # start up endpoints
         start_delay = 0
         for bootstrap in settings.SEED_LIST:
             host, port = bootstrap.split(":")
-            reactor.callLater(start_delay, self.SetupConnection, host, port)
+            setupConnDeferred = task.deferLater(reactor, start_delay, self.SetupConnection, host, port)
+            setupConnDeferred.addErrback(self.onSetupConnectionErr)
             start_delay += 1
 
         # check in on peers every 4 mins
-        self.peer_check_loop = task.LoopingCall(self.PeerCheckLoop)
-        self.peer_check_loop.start(240, now=False)
+        peer_check_loop = task.LoopingCall(self.PeerCheckLoop)
+        self.peer_loop_deferred = peer_check_loop.start(240, now=False)
+        self.peer_loop_deferred.addErrback(self.OnPeerLoopError)
 
         if settings.ACCEPT_INCOMING_PEERS:
             reactor.listenTCP(settings.NODE_PORT, NeoClientFactory(incoming_client=True))
@@ -159,21 +159,28 @@ class NodeLeader:
         addr = '%s:%s' % (host, port)
         if addr not in self.ADDRS and len(self.Peers) < settings.CONNECTED_PEER_MAX and addr not in self.DEAD_ADDRS:
             self.ADDRS.append(addr)
-            reactor.callLater(index * 10, self.SetupConnection, host, port)
+            setupConnDeferred = task.deferLater(reactor, index * 10, self.SetupConnection, host, port)
+            setupConnDeferred.addErrback(self.onSetupConnectionErr)
 
     def SetupConnection(self, host, port):
         if len(self.Peers) < settings.CONNECTED_PEER_MAX:
-            reactor.connectTCP(host, int(port), NeoClientFactory(), timeout=120)
+            try:
+                reactor.connectTCP(host, int(port), NeoClientFactory(), timeout=120)
+            except Exception as e:
+                logger.error("Could not connect TCP to %s:%s " % (host, port))
 
     def Shutdown(self):
         """Disconnect all connected peers."""
-        if self.peer_check_loop:
-            self.peer_check_loop.stop()
-            self.peer_check_loop = None
+        if self.peer_loop_deferred:
+            self.peer_loop_deferred.cancel()
+            self.peer_loop_deferred = None
 
         for p in self.Peers:
             p.Disconnect()
 
+    def OnPeerLoopError(self, err):
+        logger.debug("Error on Peer check loop %s " % err)
+
     def AddConnectedPeer(self, peer):
         """
         Add a new connect peer to the known peers list.
@@ -203,6 +210,9 @@ class NodeLeader:
         if peer in self.Peers:
             self.Peers.remove(peer)
 
+    def onSetupConnectionErr(self, err):
+        logger.debug("On setup connection error! %s" % err)
+
     def PeerCheckLoop(self):
         # often times things will get stuck on 1 peer so
         # every so often we will try to reconnect to peers
@@ -215,12 +225,14 @@ class NodeLeader:
         for addr in self.ADDRS:
             if addr not in connected and len(self.Peers) < settings.CONNECTED_PEER_MAX and addr not in self.DEAD_ADDRS:
                 host, port = addr.split(":")
-                reactor.callLater(start_delay, self.SetupConnection, host, port)
+                setupConnDeferred = task.deferLater(reactor, start_delay, self.SetupConnection, host, port)
+                setupConnDeferred.addErrback(self.onSetupConnectionErr)
+
                 start_delay += 1
 
     def ResetBlockRequestsAndCache(self):
         """Reset the block request counter and its cache."""
-        logger.debug("Resseting Block requests")
+        logger.debug("Resetting Block requests")
         self.MissionsGlobal = []
         BC.Default().BlockSearchTries = 0
         for p in self.Peers:
@@ -313,7 +325,6 @@ class NodeLeader:
             pass
 
         relayed = self.RelayDirectly(inventory)
-        # self.
         return relayed
 
     def GetTransaction(self, hash):
