@@ -68,7 +68,7 @@ def InvokeContract(wallet, tx, fee=Fixed8.Zero(), from_addr=None, owners=None):
 
             relayed = False
 
-#            print("SENDING TX: %s " % json.dumps(wallet_tx.ToJson(), indent=4))
+            #            print("SENDING TX: %s " % json.dumps(wallet_tx.ToJson(), indent=4))
 
             relayed = NodeLeader.Instance().Relay(wallet_tx)
 
@@ -91,7 +91,6 @@ def InvokeContract(wallet, tx, fee=Fixed8.Zero(), from_addr=None, owners=None):
 
 
 def InvokeWithTokenVerificationScript(wallet, tx, token, fee=Fixed8.Zero(), invoke_attrs=None):
-
     wallet_tx = wallet.MakeTransaction(tx=tx, fee=fee, use_standard=True)
 
     if wallet_tx:
@@ -149,7 +148,6 @@ def InvokeWithTokenVerificationScript(wallet, tx, token, fee=Fixed8.Zero(), invo
 def TestInvokeContract(wallet, args, withdrawal_tx=None,
                        parse_params=True, from_addr=None,
                        min_fee=DEFAULT_MIN_FEE, invoke_attrs=None, owners=None):
-
     BC = GetBlockchain()
 
     contract = BC.GetContract(args[0])
@@ -206,7 +204,6 @@ def TestInvokeContract(wallet, args, withdrawal_tx=None,
         outputs = []
 
         if neo_to_attach:
-
             output = TransactionOutput(AssetId=Blockchain.SystemShare().Hash,
                                        Value=neo_to_attach,
                                        script_hash=contract.Code.ScriptHash(),
@@ -214,7 +211,6 @@ def TestInvokeContract(wallet, args, withdrawal_tx=None,
             outputs.append(output)
 
         if gas_to_attach:
-
             output = TransactionOutput(AssetId=Blockchain.SystemCoin().Hash,
                                        Value=gas_to_attach,
                                        script_hash=contract.Code.ScriptHash())
@@ -268,13 +264,11 @@ def test_invoke(script, wallet, outputs, withdrawal_tx=None,
 
     bc = GetBlockchain()
 
-    sn = bc._db.snapshot()
-
-    accounts = DBCollection(bc._db, sn, DBPrefix.ST_Account, AccountState)
-    assets = DBCollection(bc._db, sn, DBPrefix.ST_Asset, AssetState)
-    validators = DBCollection(bc._db, sn, DBPrefix.ST_Validator, ValidatorState)
-    contracts = DBCollection(bc._db, sn, DBPrefix.ST_Contract, ContractState)
-    storages = DBCollection(bc._db, sn, DBPrefix.ST_Storage, StorageItem)
+    accounts = DBCollection(bc._db, DBPrefix.ST_Account, AccountState)
+    assets = DBCollection(bc._db, DBPrefix.ST_Asset, AssetState)
+    validators = DBCollection(bc._db, DBPrefix.ST_Validator, ValidatorState)
+    contracts = DBCollection(bc._db, DBPrefix.ST_Contract, ContractState)
+    storages = DBCollection(bc._db, DBPrefix.ST_Storage, StorageItem)
 
     # if we are using a withdrawal tx, don't recreate the invocation tx
     # also, we don't want to reset the inputs / outputs
@@ -334,7 +328,7 @@ def test_invoke(script, wallet, outputs, withdrawal_tx=None,
         testMode=True
     )
 
-    engine.LoadScript(wallet_tx.Script, False)
+    engine.LoadScript(wallet_tx.Script)
 
     try:
         success = engine.Execute()
@@ -372,7 +366,7 @@ def test_invoke(script, wallet, outputs, withdrawal_tx=None,
             wallet_tx.outputs = outputs
             wallet_tx.Attributes = [] if invoke_attrs is None else deepcopy(invoke_attrs)
 
-            return wallet_tx, net_fee, engine.EvaluationStack.Items, engine.ops_processed
+            return wallet_tx, net_fee, engine.ResultStack.Items, engine.ops_processed
 
         # this allows you to to test invocations that fail
         else:
@@ -391,18 +385,15 @@ def test_deploy_and_invoke(deploy_script, invoke_args, wallet,
                            debug_map=None, invoke_attrs=None, owners=None):
     bc = GetBlockchain()
 
-    sn = bc._db.snapshot()
-
-    accounts = DBCollection(bc._db, sn, DBPrefix.ST_Account, AccountState)
-    assets = DBCollection(bc._db, sn, DBPrefix.ST_Asset, AssetState)
-    validators = DBCollection(bc._db, sn, DBPrefix.ST_Validator, ValidatorState)
-    contracts = DBCollection(bc._db, sn, DBPrefix.ST_Contract, ContractState)
-    storages = DBCollection(bc._db, sn, DBPrefix.ST_Storage, StorageItem)
+    accounts = DBCollection(bc._db, DBPrefix.ST_Account, AccountState)
+    assets = DBCollection(bc._db, DBPrefix.ST_Asset, AssetState)
+    validators = DBCollection(bc._db, DBPrefix.ST_Validator, ValidatorState)
+    contracts = DBCollection(bc._db, DBPrefix.ST_Contract, ContractState)
+    storages = DBCollection(bc._db, DBPrefix.ST_Storage, StorageItem)
 
     if settings.USE_DEBUG_STORAGE:
         debug_storage = DebugStorage.instance()
-        debug_sn = debug_storage.db.snapshot()
-        storages = DBCollection(debug_storage.db, debug_sn, DBPrefix.ST_Storage, StorageItem)
+        storages = DBCollection(debug_storage.db, DBPrefix.ST_Storage, StorageItem)
         storages.DebugStorage = True
 
     dtx = InvocationTransaction()
@@ -438,7 +429,7 @@ def test_deploy_and_invoke(deploy_script, invoke_args, wallet,
         testMode=True
     )
 
-    engine.LoadScript(dtx.Script, False)
+    engine.LoadScript(dtx.Script)
 
     # first we will execute the test deploy
     # then right after, we execute the test invoke
@@ -447,7 +438,7 @@ def test_deploy_and_invoke(deploy_script, invoke_args, wallet,
 
     if d_success:
 
-        items = engine.EvaluationStack.Items
+        items = engine.ResultStack.Items
 
         contract_state = None
         for i in items:
@@ -565,7 +556,7 @@ def test_deploy_and_invoke(deploy_script, invoke_args, wallet,
         )
 
         engine.invocation_args = invoke_args
-        engine.LoadScript(itx.Script, False)
+        engine.LoadScript(itx.Script)
         engine.LoadDebugInfoForScriptHash(debug_map, shash.Data)
 
         i_success = engine.Execute()
@@ -596,7 +587,7 @@ def test_deploy_and_invoke(deploy_script, invoke_args, wallet,
             # set the amount of gas the tx will need
             itx.Gas = consumed
             itx.Attributes = []
-            result = engine.EvaluationStack.Items
+            result = engine.ResultStack.Items
             return itx, result, total_ops, engine
         else:
             print("error executing invoke contract...")
@@ -611,7 +602,6 @@ def test_deploy_and_invoke(deploy_script, invoke_args, wallet,
 
 def gather_signatures(context, itx, owners):
     do_exit = False
-    print("owners %s " % owners)
     print("\n\n*******************\n")
     print("Gather Signatures for Transaction:\n%s " % json.dumps(itx.ToJson(), indent=4))
     print("Please use a client to sign the following: %s " % itx.GetHashData())
