@@ -9,16 +9,22 @@ import shutil
 
 
 class UserWalletTestCase(WalletFixtureTestCase):
-    wallet_1_script_hash = UInt160(data=b'S\xefB\xc8\xdf!^\xbeZ|z\xe8\x01\xcb\xc3\xac/\xacI)')
+    wallet_1_script_hash = UInt160(data=b'\x1c\xc9\xc0\\\xef\xff\xe6\xcd\xd7\xb1\x82\x81j\x91R\xec!\x8d.\xc0')
 
-    wallet_1_addr = 'APRgMZHZubii29UXF9uFa6sohrsYupNAvx'
+    wallet_1_addr = 'AJQ6FoaSXDFzA6wLnyZ1nFN7SGSN2oNTc3'
 
-    wallet_3_script_hash = UInt160(data=b'\xa6\xc5\x9d\xeb\xf0\xd7(\xbd\x14\x89\xcd\xb9\xd9{\xd1\x90\xcb\x0b\xdch')
+    wallet_2_script_hash = UInt160(data=b'\x08t/\\P5\xac-\x0b\x1c\xb4\x94tIyBu\x7f1*')
 
-    wallet_3_addr = 'AWygZ1B5c3GDiLL6u5bHSVU45Ya1SVGX9P'
+    wallet_2_addr = 'AGYaEi3W6ndHPUmW7T12FFfsbQ6DWymkEm'
+
+    wallet_3_script_hash = UInt160(data=b'\xc4\xc1\xb0\xcf\xa8\x7f\xcb\xacE\x98W0\x16d\x11\x03]\xdf\xed#')
+
+    wallet_3_addr = 'AZiE7xfyJALW7KmADWtCJXGGcnduYhGiCX'
 
     _wallet1 = None
 
+    _wallet2 = None
+
     _wallet3 = None
 
     @property
@@ -37,6 +43,14 @@ class UserWalletTestCase(WalletFixtureTestCase):
                                            to_aes_key(UserWalletTestCase.wallet_1_pass()))
         return cls._wallet1
 
+    @classmethod
+    def GetWallet2(cls, recreate=False):
+        if cls._wallet2 is None or recreate:
+            shutil.copyfile(cls.wallet_2_path(), cls.wallet_2_dest())
+            cls._wallet2 = UserWallet.Open(UserWalletTestCase.wallet_2_dest(),
+                                           to_aes_key(UserWalletTestCase.wallet_2_pass()))
+        return cls._wallet2
+
     @classmethod
     def GetWallet3(cls, recreate=False):
         if cls._wallet3 is None or recreate:
@@ -47,7 +61,7 @@ class UserWalletTestCase(WalletFixtureTestCase):
 
     def test_1_no_available_claim(self):
 
-        wallet = self.GetWallet1()
+        wallet = self.GetWallet3()
 
         unspents = wallet.FindUnspentCoinsByAsset(self.NEO)
 
@@ -55,7 +69,7 @@ class UserWalletTestCase(WalletFixtureTestCase):
 
         unavailable_bonus = wallet.GetUnavailableBonus()
 
-        self.assertEqual(Fixed8.FromDecimal(0.144727), unavailable_bonus)
+        self.assertEqual(Fixed8.FromDecimal(0.0002685), unavailable_bonus) 
 
         unclaimed_coins = wallet.GetUnclaimedCoins()
 
@@ -67,27 +81,27 @@ class UserWalletTestCase(WalletFixtureTestCase):
 
     def test_2_wallet_with_claimable_gas(self):
 
-        wallet = self.GetWallet3()
+        wallet = self.GetWallet1()
 
         unspents = wallet.FindUnspentCoinsByAsset(self.NEO)
 
-        self.assertEqual(2, len(unspents))
+        self.assertEqual(1, len(unspents))
 
         unavailable_bonus = wallet.GetUnavailableBonus()
 
-        self.assertEqual(Fixed8.FromDecimal(0.13324017), unavailable_bonus)
+        self.assertEqual(Fixed8.FromDecimal(0.000601), unavailable_bonus)
 
         unclaimed_coins = wallet.GetUnclaimedCoins()
 
-        self.assertEqual(4, len(unclaimed_coins))
+        self.assertEqual(1, len(unclaimed_coins))
 
         available_bonus = wallet.GetAvailableClaimTotal()
 
-        self.assertEqual(Fixed8.FromDecimal(0.0048411), available_bonus)
+        self.assertEqual(Fixed8.FromDecimal(0.000288), available_bonus)
 
     def test_3_wallet_no_claimable_gas(self):
 
-        wallet = self.GetWallet1()
+        wallet = self.GetWallet3()
 
         result = ClaimGas(wallet, require_password=False)
 
@@ -95,7 +109,7 @@ class UserWalletTestCase(WalletFixtureTestCase):
 
     def test_4_wallet_claim_ok(self):
 
-        wallet = self.GetWallet3()
+        wallet = self.GetWallet1()
 
         claim = ClaimGas(wallet, require_password=False, args=['1'])
 
@@ -103,26 +117,26 @@ class UserWalletTestCase(WalletFixtureTestCase):
 
     def test_5_wallet_claim_ok(self):
 
-        wallet = self.GetWallet3()
+        wallet = self.GetWallet2()
 
         claim = ClaimGas(wallet, require_password=False)
 
         self.assertTrue(claim)
 
-    def test_block_150000_sysfee(self):
+    def test_block_12248_sysfee(self):
 
-        fee = Blockchain.Default().GetSysFeeAmountByHeight(150000)
+        fee = Blockchain.Default().GetSysFeeAmountByHeight(12248)
 
-        self.assertEqual(fee, 1230)
+        self.assertEqual(fee, 2050)
 
-    def test_block_347809_sysfee(self):
+    def test_block_12323_sysfee(self):
 
-        fee = Blockchain.Default().GetSysFeeAmountByHeight(347809)
+        fee = Blockchain.Default().GetSysFeeAmountByHeight(12323)
 
-        self.assertEqual(fee, 39134)
+        self.assertEqual(fee, 2540)
 
-    def test_block_747809_sysfee(self):
+    def test_block_11351_sysfee(self):
 
-        fee = Blockchain.Default().GetSysFeeAmountByHeight(747809)
+        fee = Blockchain.Default().GetSysFeeAmountByHeight(11351)
 
-        self.assertEqual(fee, 472954)
+        self.assertEqual(fee, 1560)
