@@ -14,15 +14,15 @@ from neo.IO.MemoryStream import StreamManager
 
 class UserWalletTestCase(WalletFixtureTestCase):
 
-    wallet_1_script_hash = UInt160(data=b'S\xefB\xc8\xdf!^\xbeZ|z\xe8\x01\xcb\xc3\xac/\xacI)')
+    wallet_1_script_hash = UInt160(data=b'\x1c\xc9\xc0\\\xef\xff\xe6\xcd\xd7\xb1\x82\x81j\x91R\xec!\x8d.\xc0')
 
-    wallet_1_addr = 'APRgMZHZubii29UXF9uFa6sohrsYupNAvx'
+    wallet_1_addr = 'AJQ6FoaSXDFzA6wLnyZ1nFN7SGSN2oNTc3'
 
-    import_watch_addr = UInt160(data=b'\xaf\x12\xa8h{\x14\x94\x8b\xc4\xa0\x08\x12\x8aU\nci[\xc1\xa5')
-    watch_addr_str = 'AXjaFSP23Jkbe6Pk9pPGT6NBDs1HVdqaXK'
+    import_watch_addr = UInt160(data=b'\x08t/\\P5\xac-\x0b\x1c\xb4\x94tIyBu\x7f1*')
+    watch_addr_str = 'AGYaEi3W6ndHPUmW7T12FFfsbQ6DWymkEm'
     _wallet1 = None
 
-    token_hash_str = 'f8d448b227991cf07cb96a6f9c0322437f1599b9'
+    token_hash_str = '31730cc9a1844891a3bafd1aa929a4142860d8d3'
 
     @property
     def GAS(self):
@@ -53,16 +53,16 @@ class UserWalletTestCase(WalletFixtureTestCase):
 
         wallet = self.GetWallet1()
 
-        self.assertEqual(len(wallet.GetTokens()), 0)
+        self.assertEqual(len(wallet.GetTokens()), 1)
 
         ImportToken(wallet, self.token_hash_str)
 
         token = list(wallet.GetTokens().values())[0]
 
-        self.assertEqual(token.name, 'NEP5 Standard')
-        self.assertEqual(token.symbol, 'NEP5')
+        self.assertEqual(token.name, 'NEX Template V4')
+        self.assertEqual(token.symbol, 'NXT4')
         self.assertEqual(token.decimals, 8)
-        self.assertEqual(token.Address, 'AYhE3Svuqdfh1RtzvE8hUhNR7HSpaSDFQg')
+        self.assertEqual(token.Address, 'Ab61S1rk2VtCVd3NtGNphmBckWk4cfBdmB')
 
     def test_2_token_balance(self):
 
@@ -72,7 +72,7 @@ class UserWalletTestCase(WalletFixtureTestCase):
 
         balance = wallet.GetBalance(token)
 
-        self.assertEqual(balance, 9999)
+        self.assertEqual(balance, 2499000)
 
     def test_3_token_allowance(self):
 
@@ -196,7 +196,6 @@ class UserWalletTestCase(WalletFixtureTestCase):
 
         self.assertFalse(result)
 
-    # this will fail since this token doesn't have a mint method
     def test_8_mint(self):
 
         wallet = self.GetWallet1(recreate=True)
@@ -211,7 +210,7 @@ class UserWalletTestCase(WalletFixtureTestCase):
 
         mint = token_mint(wallet, args, prompt_passwd=False)
 
-        self.assertFalse(mint)
+        self.assertTrue(mint)
 
     def test_token_serialize(self):
 
@@ -225,4 +224,4 @@ class UserWalletTestCase(WalletFixtureTestCase):
         writer = BinaryWriter(stream)
         token.Serialize(writer)
 
-        self.assertEqual(b'0d4e455035205374616e64617264044e45503508', stream.ToArray())
+        self.assertEqual(b'0f4e45582054656d706c617465205634044e58543408', stream.ToArray())
