@@ -14,11 +14,18 @@ class VerifiableTestCase(NeoTestCase):
 
     @classmethod
     def setUpClass(self):
+
+        Blockchain.DeregisterBlockchain()
+
         os.makedirs(self.LEVELDB_TESTPATH, exist_ok=True)
         self._blockchain = LevelDBBlockchain(path=self.LEVELDB_TESTPATH, skip_version_check=True)
         Blockchain.RegisterBlockchain(self._blockchain)
 
     @classmethod
     def tearDownClass(cls):
-        cls._blockchain.Dispose()
+
+        Blockchain.Default().DeregisterBlockchain()
+        if cls._blockchain is not None:
+            cls._blockchain.Dispose()
+
         shutil.rmtree(cls.LEVELDB_TESTPATH)
