@@ -7,7 +7,6 @@ from neo.SmartContract import StackItemType
 
 
 class CollectionMixin:
-
     IsSynchronized = False
     SyncRoot = None
 
@@ -59,14 +58,14 @@ class StackItem(EquatableMixin):
     def Serialize(self, writer):
         pass
 
-#    def Deserialize(self, reader):
-#        pass
+    #    def Deserialize(self, reader):
+    #        pass
 
     def __hash__(self):
         hash = 17
         for b in self.GetByteArray():
             hash = hash * 31 + b
-#        print("hash code %s " % hash)
+        #        print("hash code %s " % hash)
         return hash
 
     def __str__(self):
@@ -133,12 +132,11 @@ class StackItem(EquatableMixin):
         elif typ is list:
             return Array(value)
 
-#        logger.debug("Could not create stack item for vaule %s %s " % (typ, value))
+        #        logger.debug("Could not create stack item for vaule %s %s " % (typ, value))
         return value
 
 
 class Array(StackItem, CollectionMixin):
-
     _array = None  # a list of stack items
 
     @property
@@ -219,7 +217,6 @@ class Array(StackItem, CollectionMixin):
 
 
 class Boolean(StackItem):
-
     TRUE = bytearray([1])
     FALSE = bytearray([0])
 
@@ -228,6 +225,10 @@ class Boolean(StackItem):
     def __init__(self, value):
         self._value = value
 
+    @property
+    def Count(self):
+        return 1
+
     def Equals(self, other):
         if other is None:
             return False
@@ -257,12 +258,15 @@ class Boolean(StackItem):
 
 
 class ByteArray(StackItem):
-
     _value = None
 
     def __init__(self, value):
         self._value = value
 
+    @property
+    def Count(self):
+        return len(self._value)
+
     def Equals(self, other):
         if other is None:
             return False
@@ -300,11 +304,11 @@ class ByteArray(StackItem):
     def __str__(self):
         return self.GetString()
 
+
 #
 
 
 class Integer(StackItem):
-
     _value = None
 
     def __init__(self, value):
@@ -312,6 +316,10 @@ class Integer(StackItem):
             raise Exception("Must be big integer instance")
         self._value = value
 
+    @property
+    def Count(self):
+        return 1
+
     def Equals(self, other):
         if other is None:
             return False
@@ -341,7 +349,6 @@ class Integer(StackItem):
 
 
 class InteropInterface(StackItem):
-
     _object = None
 
     def __init__(self, value):
@@ -422,7 +429,6 @@ class Struct(Array):
 
 
 class Map(StackItem, CollectionMixin):
-
     _dict = None
 
     def __init__(self, dict=None):
@@ -514,7 +520,6 @@ class Map(StackItem, CollectionMixin):
 
 
 class InteropService:
-
     _dictionary = {}
 
     def __init__(self):
@@ -541,21 +546,21 @@ class InteropService:
 
     @staticmethod
     def GetScriptContainer(engine):
-        engine.EvaluationStack.PushT(StackItem.FromInterface(engine.ScriptContainer))
+        engine.CurrentContext.EvaluationStack.PushT(StackItem.FromInterface(engine.ScriptContainer))
         return True
 
     @staticmethod
     def GetExecutingScriptHash(engine):
-        engine.EvaluationStack.PushT(engine.CurrentContext.ScriptHash())
+        engine.CurrentContext.EvaluationStack.PushT(engine.CurrentContext.ScriptHash())
         return True
 
     @staticmethod
     def GetCallingScriptHash(engine):
-        engine.EvaluationStack.PushT(engine.CallingContext.ScriptHash())
+        engine.CurrentContext.EvaluationStack.PushT(engine.CallingContext.ScriptHash())
         return True
 
     @staticmethod
     def GetEntryScriptHash(engine):
 
-        engine.EvaluationStack.PushT(engine.EntryContext.ScriptHash())
+        engine.CurrentContext.EvaluationStack.PushT(engine.EntryContext.ScriptHash())
         return True
