@@ -86,7 +86,7 @@ class PromptInterface:
 
     go_on = True
 
-    _walletdb_loop = None
+    wallet_loop_deferred = None
 
     Wallet = None
 
@@ -111,6 +111,7 @@ class PromptInterface:
                 'config maxpeers {num_peers}',
                 'config node-requests {reqsize} {queuesize}',
                 'config node-requests {slow/normal/fast}',
+                'config compiler-nep8 {on/off}',
                 'build {path/to/file.py} (test {params} {returntype} {needs_storage} {needs_dynamic_invoke} {is_payable} [{test_params} or --i]) --no-parse-addr (parse address strings to script hash bytearray)',
                 'load_run {path/to/file.avm} (test {params} {returntype} {needs_storage} {needs_dynamic_invoke} {is_payable} [{test_params} or --i]) --no-parse-addr (parse address strings to script hash bytearray)',
                 'import wif {wif}',
@@ -203,7 +204,7 @@ class PromptInterface:
                                 'claim', 'migrate', 'rebuild', 'create_addr', 'delete_addr',
                                 'delete_token', 'alias', 'unspent', 'split', 'close',
                                 'withdraw_reqest', 'holds', 'completed', 'cancel', 'cleanup',
-                                'all', 'debugstorage']
+                                'all', 'debugstorage', 'compiler-nep8', ]
 
         if self.Wallet:
             for addr in self.Wallet.Addresses:
@@ -312,12 +313,15 @@ class PromptInterface:
                 print("Please specify a path")
 
     def start_wallet_loop(self):
-        self._walletdb_loop = task.LoopingCall(self.Wallet.ProcessBlocks)
-        self._walletdb_loop.start(1)
+        if self.wallet_loop_deferred:
+            self.stop_wallet_loop()
+        walletdb_loop = task.LoopingCall(self.Wallet.ProcessBlocks)
+        self.wallet_loop_deferred = walletdb_loop.start(1)
+        self.wallet_loop_deferred.addErrback(self.on_looperror)
 
     def stop_wallet_loop(self):
-        self._walletdb_loop.stop()
-        self._walletdb_loop = None
+        self.wallet_loop_deferred.cancel()
+        self.wallet_loop_deferred = None
 
     def do_close_wallet(self):
         if self.Wallet:
@@ -643,7 +647,7 @@ class PromptInterface:
             block = Blockchain.Default().GetBlock(item)
 
             if block is not None:
-
+                block.LoadTransactions()
                 bjson = json.dumps(block.ToJson(), indent=4)
                 tokens = [("class:number", bjson)]
                 print_formatted_text(FormattedText(tokens), style=self.token_style)
@@ -954,14 +958,32 @@ class PromptInterface:
 
             else:
                 print("Maintaining current number of maxpeers")
+        elif what == 'compiler-nep8':
+            c1 = get_arg(args, 1)
+            if c1 is not None:
+                c1 = c1.lower()
+                if c1 == 'on' or c1 == '1':
+                    print("Compiler NEP8 instructions on")
+                    settings.COMPILER_NEP_8 = True
+                elif c1 == 'off' or c1 == '0':
+                    print("Compiler NEP8 instructions off")
+                    settings.COMPILER_NEP_8 = False
+                else:
+                    print("Cannot configure compiler NEP8 instructions. Please specify on|off")
+            else:
+                print("Cannot configure compiler NEP8 instructions. Please specify on|off")
 
         else:
             print(
-                "Cannot configure %s try 'config sc-events on|off', 'config debug on|off', 'config sc-debug-notify on|off', 'config vm-log on|off', or 'config maxpeers {num_peers}'" % what)
+                "Cannot configure %s try 'config sc-events on|off', 'config debug on|off', 'config sc-debug-notify on|off', 'config vm-log on|off', config compiler-nep8 on|off, or 'config maxpeers {num_peers}'" % what)
+
+    def on_looperror(self, err):
+        logger.debug("On DB loop error! %s " % err)
 
     def run(self):
         dbloop = task.LoopingCall(Blockchain.Default().PersistBlocks)
-        dbloop.start(.1)
+        dbloop_deferred = dbloop.start(.1)
+        dbloop_deferred.addErrback(self.on_looperror)
 
         tokens = [("class:neo", 'NEO'), ("class:default", ' cli. Type '),
                   ("class:command", '\'help\' '), ("class:default", 'to get started')]
@@ -987,6 +1009,8 @@ class PromptInterface:
             except KeyboardInterrupt:
                 # Control-C pressed: do nothing
                 continue
+            except Exception as e:
+                logger.error("Exception handling input: %s " % e)
 
             try:
                 command, arguments = self.input_parser.parse_input(result)
@@ -1145,8 +1169,9 @@ def main():
     cli = PromptInterface(fn_prompt_history)
 
     # Run things
-    #    reactor.suggestThreadPoolSize(15)
+
     reactor.callInThread(cli.run)
+
     NodeLeader.Instance().Start()
 
     # reactor.run() is blocking, until `quit()` is called which stops the reactor.
